# -*- coding: UTF-8 -*-
from setuphelpers import *
from setupdevhelpers import *
import waptlicences
import json
import requests
import re

url = "https://www.aplim.fr/Documentation.aspx/GetDirectoryAndFiles"
target_text = "Moteur d'Exécution"
latest_bin = "charlemagne_moteur.exe"

def update_package():

    def find_charlemagne_download(data, target_text):
        
        downloads = next((item['nodes'] for item in data if item.get('text') == "Téléchargements"), [])
        charlemagne_item = next((item for item in downloads if item.get('text') == "Charlemagne"), None)

        if charlemagne_item:
            return next((item for item in charlemagne_item.get('nodes', []) if target_text in item.get('text', '')), None)
        
        return None

    def contains_letters(chaine):
        return any(c.isalpha() for c in chaine)

    # Declaring local variables
    package_updated = False
    proxies = get_proxies_from_wapt_console()
    if not proxies:
        proxies = get_proxies()

    app_name = control.name

    headers = {
        "Accept": "application/json",
        "Content-Type": "application/json",
        "User-Agent": "WAPT"
    }

    try:
        response = requests.get(url, headers=headers, proxies=proxies)
        response.raise_for_status()
        response = response.json()
    except Exception as e:
        print(f"Request error: {e}")
        return package_updated
    
    decoded_data = json.loads(response['d'])
    data = find_charlemagne_download(decoded_data, target_text)

    version = re.search(rf"{target_text} - (\d+(?:\.\d+)+)", data['text']).group(1)
    
    download_url = data['href']

    print(f"Latest {app_name} version is: {version}")
    print(f"Download url is: {download_url}")

    # Before Aplim set up a bad naming scheme for its applications (Ex: 21.B , 9.2A ...)
    # so we check if this is not the case
    if contains_letters(version):
        print("For the moment, this application cannot be packaged due to incorrect version naming...")
        return package_updated

    for f in glob.glob('*.exe'):
        if Version(get_version_from_binary(f)) < Version(version):
            remove_file(f)

    if not isfile(latest_bin):
        print(f"Downloading: {latest_bin}")
        package_updated = True
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print(f"Binary is present: {latest_bin}")

    expected_issuer = 'Aplim'
    sign_name = waptlicences.check_exe_signing_certificate(latest_bin)[0]
    if sign_name != expected_issuer:
        error(f'Bad issuer {sign_name} != {expected_issuer} ')

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print(f"Software version updated (from: {control.get_software_version()} to: {version})")
        package_updated = True
    else:
        print(f"Software version up-to-date ({version})")
        
    control.set_software_version(get_version_from_binary(latest_bin))
    control.save_control_to_wapt()


    # Validating or not update-package-sources
    return package_updated
